@@ -199,7 +199,6 @@ g_mount_spec_ref (GMountSpec *spec)
   return spec;
 }
 
-
 void
 g_mount_spec_unref (GMountSpec *spec)
 {
@@ -456,23 +455,126 @@ g_mount_spec_to_string (GMountSpec *spec)
 {
   GString *str;
   int i;
+  gboolean first;
 
   if (spec == NULL)
     return g_strdup ("(null)");
 
-  str = g_string_new ("");
+  str = g_string_new (g_mount_spec_get_type (spec));
+  g_string_append_c (str, ':');
 
+  first = TRUE;
   for (i = 0; i < spec->items->len; i++)
     {
       GMountSpecItem *item = &g_array_index (spec->items, GMountSpecItem, i);
 
-      g_string_append_printf (str, "%s='%s',", item->key, item->value);
+      if (strcmp (item->key, "type") == 0)
+	continue;
+
+      if (!first)
+	g_string_append_c (str, ',');
+      first = FALSE;
+
+      g_string_append_printf (str, "%s=", item->key);
+      g_string_append_uri_escaped (str, item->value,
+				   "$&'()*+",
+				   TRUE);
+    }
+
+  if (strcmp (spec->mount_prefix, "/") != 0)
+    {
+      g_string_append_printf (str, ",prefix=");
+      g_string_append_uri_escaped (str, spec->mount_prefix,
+				   "$&'()*+",
+				   TRUE);
     }
-  g_string_append_printf (str, "mount_prefix='%s'", spec->mount_prefix);
 
   return g_string_free (str, FALSE);
 }
 
+GMountSpec *
+g_mount_spec_new_from_string (const gchar     *str,
+                              GError         **error)
+{
+  GArray *items;
+  GMountSpec *mount_spec;
+  char **kv_pairs;
+  char *mount_prefix;
+  const char *colon;
+  GMountSpecItem item;
+  int i;
+
+  g_return_val_if_fail (str != NULL, NULL);
+
+  mount_spec = NULL;
+  mount_prefix = NULL;
+  items = g_array_new (FALSE, TRUE, sizeof (GMountSpecItem));
+
+  colon = strchr (str, ':');
+  if (colon)
+    {
+      item.key = g_strdup ("type");
+      item.value = g_strndup (str, colon - str - 1);
+      g_array_append_val (items, item);
+      str = colon + 1;
+    }
+
+  kv_pairs = g_strsplit (str, ",", 0);
+  for (i = 0; kv_pairs[i] != NULL; i++)
+    {
+      char **tokens;
+
+      tokens = g_strsplit (kv_pairs[i], "=", 0);
+      if (g_strv_length (tokens) != 2)
+        {
+          g_set_error (error,
+                       G_IO_ERROR,
+                       G_IO_ERROR_INVALID_ARGUMENT,
+                       "Encountered invalid key/value pair '%s' while decoding GMountSpec",
+                       kv_pairs[i]);
+          g_strfreev (tokens);
+          g_strfreev (kv_pairs);
+          goto fail;
+        }
+
+      item.value = g_uri_unescape_string (tokens[1], NULL);
+      if (strcmp (tokens[0], "prefix") == 0)
+        {
+          g_free (item.key);
+          mount_prefix = item.value;
+        }
+      else
+        {
+	  item.key = g_strdup (tokens[0]);
+          g_array_append_val (items, item);
+        }
+
+      g_strfreev (tokens);
+    }
+  g_strfreev (kv_pairs);
+
+  if (mount_prefix == NULL)
+    mount_prefix = g_strdup ("/");
+
+  /* this constructor takes ownership of the data we pass in */
+  mount_spec = g_mount_spec_new_from_data (items,
+                                           mount_prefix);
+
+  return mount_spec;
+
+ fail:
+  for (i = 0; i < items->len; i++)
+    {
+      GMountSpecItem *item = &g_array_index (items, GMountSpecItem, i);
+      g_free (item->key);
+      g_free (item->value);
+    }
+  g_array_free (items, TRUE);
+  g_free (mount_prefix);
+  return NULL;
+}
+
+
 char *
 g_mount_spec_canonicalize_path (const char *path)
 {
@@ -532,3 +634,15 @@ g_mount_spec_canonicalize_path (const char *path)
   
   return canon;
 }
+
+GType
+g_type_mount_spec_get_gtype (void)
+{
+  static GType type_id = 0;
+
+  if (type_id == 0)
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GMountSpec"),
+                                            (GBoxedCopyFunc) g_mount_spec_ref,
+                                            (GBoxedFreeFunc) g_mount_spec_unref);
+  return type_id;
+}
