@@ -21,7 +21,6 @@
  *          Cosimo Cecchi <cosimoc@gnome.org>
  */
 
-
 #include <config.h>
 
 #include <sys/types.h>
@@ -63,9 +62,16 @@ typedef struct {
   GIcon *icon;
   GFile *root;
   int prio;
+  gchar *unix_device_file;
   gboolean can_mount;
   gboolean can_unmount;
   gboolean can_eject;
+  gboolean can_start;
+  gboolean can_start_degraded;
+  gboolean can_stop;
+  gboolean can_poll_for_media;
+  gboolean is_media_check_automatic;
+  GDriveStartStopType start_stop_type;
   
   GDrive *drive;
   GVolume *volume;
@@ -94,6 +100,7 @@ G_DEFINE_TYPE (GVfsBackendComputer, g_vfs_backend_computer, G_VFS_TYPE_BACKEND)
 static void
 computer_file_free (ComputerFile *file)
 {
+  g_free (file->unix_device_file);
   g_free (file->filename);
   g_free (file->display_name);
   if (file->icon)
@@ -133,7 +140,13 @@ computer_file_equal (ComputerFile *a,
 
   if (a->can_mount != b->can_mount ||
       a->can_unmount != b->can_unmount ||
-      a->can_eject != b->can_eject)
+      a->can_eject != b->can_eject ||
+      a->can_start != b->can_start ||
+      a->can_start_degraded != b->can_start_degraded ||
+      a->can_stop != b->can_stop ||
+      a->can_poll_for_media != b->can_poll_for_media ||
+      a->is_media_check_automatic != b->is_media_check_automatic ||
+      a->start_stop_type != b->start_stop_type)
     return FALSE;
 
   return TRUE;
@@ -310,6 +323,9 @@ recompute_files (GVfsBackendComputer *backend)
   char *basename, *filename;
   const char *extension;
   int uniq;
+  gchar *s;
+  gchar *display_name;
+  gchar *drive_name;
 
   volume_monitor = backend->volume_monitor;
 
@@ -407,22 +423,53 @@ recompute_files (GVfsBackendComputer *backend)
 
       if (file->mount)
         {
+          if (file->drive != NULL)
+            {
+              drive_name = g_drive_get_name (file->drive);
+              s = g_mount_get_name (file->mount);
+              display_name = g_strdup_printf ("%s: %s", drive_name, s);
+              g_free (s);
+              g_free (drive_name);
+            }
+          else
+            {
+              display_name = g_mount_get_name (file->mount);
+            }
+          if (file->volume != NULL)
+            file->unix_device_file = g_volume_get_identifier (file->volume,
+                                                              G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
           file->icon = g_mount_get_icon (file->mount);
-          file->display_name = g_mount_get_name (file->mount);
+          file->display_name = display_name;
           file->root = g_mount_get_root (file->mount);
           file->can_unmount = g_mount_can_unmount (file->mount);
           file->can_eject = g_mount_can_eject (file->mount);
         }
       else if (file->volume)
         {
+          if (file->drive != NULL)
+            {
+              drive_name = g_drive_get_name (file->drive);
+              s = g_volume_get_name (file->volume);
+              display_name = g_strdup_printf ("%s: %s", drive_name, s);
+              g_free (s);
+              g_free (drive_name);
+            }
+          else
+            {
+              display_name = g_volume_get_name (file->volume);
+            }
+          file->unix_device_file = g_volume_get_identifier (file->volume,
+                                                            G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
           file->icon = g_volume_get_icon (file->volume);
-          file->display_name = g_volume_get_name (file->volume);
+          file->display_name = display_name;
           file->can_mount = g_volume_can_mount (file->volume);
           file->root = NULL;
           file->can_eject = g_volume_can_eject (file->volume);
         }
       else /* drive */
         {
+          file->unix_device_file = g_drive_get_identifier (file->drive,
+                                                            G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
           file->icon = g_drive_get_icon (file->drive);
           file->display_name = g_drive_get_name (file->drive);
           file->can_eject = g_drive_can_eject (file->drive);
@@ -431,6 +478,14 @@ recompute_files (GVfsBackendComputer *backend)
 
       if (file->drive)
         {
+          file->can_start = g_drive_can_start (file->drive);
+          file->can_start_degraded = g_drive_can_start_degraded (file->drive);
+          file->can_stop = g_drive_can_stop (file->drive);
+          file->can_poll_for_media = g_drive_can_poll_for_media (file->drive);
+          file->is_media_check_automatic = g_drive_is_media_check_automatic (file->drive);
+          file->start_stop_type = g_drive_get_start_stop_type (file->drive);
+          if (file->can_start)
+            file->can_mount = FALSE;
           basename = g_drive_get_name (file->drive);
           extension = ".drive";
         }
@@ -620,9 +675,17 @@ file_info_from_file (ComputerFile *file,
   g_file_info_set_sort_order (info, file->prio);
 
   g_file_info_set_file_type (info, G_FILE_TYPE_MOUNTABLE);
+  if (file->unix_device_file != NULL)
+    g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_MOUNTABLE_UNIX_DEVICE_FILE, file->unix_device_file);
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_MOUNT, file->can_mount);
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_UNMOUNT, file->can_unmount);
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_EJECT, file->can_eject);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_START, file->can_start);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_START_DEGRADED, file->can_start_degraded);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_STOP, file->can_stop);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_POLL, file->can_poll_for_media);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_IS_MEDIA_CHECK_AUTOMATIC, file->is_media_check_automatic);
+  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_MOUNTABLE_START_STOP_TYPE, file->start_stop_type);
 
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE, FALSE);
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE, FALSE);
@@ -754,8 +817,6 @@ mount_volume_cb (GObject *source_object,
   
   volume = G_VOLUME (source_object);
 
-  /* TODO: We're leaking the GMountOperation here */
-  
   error = NULL;
   if (g_volume_mount_finish (volume, res, &error))
     {
@@ -883,6 +944,8 @@ try_mount_mountable (GVfsBackend *backend,
       if (file->volume)
         {
           mount_op = g_mount_source_get_operation (mount_source);
+          /* free mount_op when job is completed */
+          g_object_set_data_full (G_OBJECT (job), "gvfs-backend-computer-mount-op", mount_op, g_object_unref);
           g_volume_mount (file->volume,
                           0,
                           mount_op,
@@ -944,7 +1007,7 @@ unmount_mount_cb (GObject *source_object,
   mount = G_MOUNT (source_object);
 
   error = NULL;
-  if (g_mount_unmount_finish (mount, res, &error))
+  if (g_mount_unmount_with_operation_finish (mount, res, &error))
     g_vfs_job_succeeded (G_VFS_JOB (job));
   else
     {
@@ -958,9 +1021,11 @@ static gboolean
 try_unmount_mountable (GVfsBackend *backend,
 		       GVfsJobUnmountMountable *job,
 		       const char *filename,
-		       GMountUnmountFlags flags)
+		       GMountUnmountFlags flags,
+                       GMountSource *mount_source)
 {
   ComputerFile *file;
+  GMountOperation *mount_op;
 
   file = lookup (G_VFS_BACKEND_COMPUTER (backend),
                  G_VFS_JOB (job), filename);
@@ -973,11 +1038,15 @@ try_unmount_mountable (GVfsBackend *backend,
     {
       if (file->mount)
         {
-          g_mount_unmount (file->mount,
-                         flags,
-                         G_VFS_JOB (job)->cancellable,
-                         unmount_mount_cb,
-                         job);
+          mount_op = g_mount_source_get_operation (mount_source);
+          /* free mount_op when job is completed */
+          g_object_set_data_full (G_OBJECT (job), "gvfs-backend-computer-mount-op", mount_op, g_object_unref);
+          g_mount_unmount_with_operation (file->mount,
+                                          flags,
+                                          mount_op,
+                                          G_VFS_JOB (job)->cancellable,
+                                          unmount_mount_cb,
+                                          job);
         }
       else
         {
@@ -1002,7 +1071,7 @@ eject_mount_cb (GObject *source_object,
   mount = G_MOUNT (source_object);
 
   error = NULL;
-  if (g_mount_eject_finish (mount, res, &error))
+  if (g_mount_eject_with_operation_finish (mount, res, &error))
     g_vfs_job_succeeded (G_VFS_JOB (job));
   else
     {
@@ -1023,7 +1092,7 @@ eject_volume_cb (GObject *source_object,
   volume = G_VOLUME (source_object);
 
   error = NULL;
-  if (g_volume_eject_finish (volume, res, &error))
+  if (g_volume_eject_with_operation_finish (volume, res, &error))
     g_vfs_job_succeeded (G_VFS_JOB (job));
   else
     {
@@ -1045,7 +1114,7 @@ eject_drive_cb (GObject *source_object,
   drive = G_DRIVE (source_object);
 
   error = NULL;
-  if (g_drive_eject_finish (drive, res, &error))
+  if (g_drive_eject_with_operation_finish (drive, res, &error))
     g_vfs_job_succeeded (G_VFS_JOB (job));
   else
     {
@@ -1058,9 +1127,11 @@ static gboolean
 try_eject_mountable (GVfsBackend *backend,
                      GVfsJobUnmountMountable *job,
                      const char *filename,
-                     GMountUnmountFlags flags)
+                     GMountUnmountFlags flags,
+                     GMountSource *mount_source)
 {
   ComputerFile *file;
+  GMountOperation *mount_op;
 
   file = lookup (G_VFS_BACKEND_COMPUTER (backend),
                  G_VFS_JOB (job), filename);
@@ -1073,27 +1144,39 @@ try_eject_mountable (GVfsBackend *backend,
     {
       if (file->mount)
         {
-          g_mount_eject (file->mount,
-                         flags,
-                         G_VFS_JOB (job)->cancellable,
-                         eject_mount_cb,
-                         job);
+          mount_op = g_mount_source_get_operation (mount_source);
+          /* free mount_op when job is completed */
+          g_object_set_data_full (G_OBJECT (job), "gvfs-backend-computer-mount-op", mount_op, g_object_unref);
+          g_mount_eject_with_operation (file->mount,
+                                        flags,
+                                        mount_op,
+                                        G_VFS_JOB (job)->cancellable,
+                                        eject_mount_cb,
+                                        job);
         }
       else if (file->volume)
         {
-          g_volume_eject (file->volume,
-                          flags,
-                          G_VFS_JOB (job)->cancellable,
-                          eject_volume_cb,
-                          job);
+          mount_op = g_mount_source_get_operation (mount_source);
+          /* free mount_op when job is completed */
+          g_object_set_data_full (G_OBJECT (job), "gvfs-backend-computer-mount-op", mount_op, g_object_unref);
+          g_volume_eject_with_operation (file->volume,
+                                         flags,
+                                         mount_op,
+                                         G_VFS_JOB (job)->cancellable,
+                                         eject_volume_cb,
+                                         job);
         }
       else if (file->drive)
         {
-          g_drive_eject (file->drive,
-                         flags,
-                         G_VFS_JOB (job)->cancellable,
-                         eject_drive_cb,
-                         job);
+          mount_op = g_mount_source_get_operation (mount_source);
+          /* free mount_op when job is completed */
+          g_object_set_data_full (G_OBJECT (job), "gvfs-backend-computer-mount-op", mount_op, g_object_unref);
+          g_drive_eject_with_operation (file->drive,
+                                        flags,
+                                        mount_op,
+                                        G_VFS_JOB (job)->cancellable,
+                                        eject_drive_cb,
+                                        job);
         }
       else
         {
@@ -1106,6 +1189,215 @@ try_eject_mountable (GVfsBackend *backend,
   return TRUE;
 }
 
+
+static void
+drive_start_cb (GObject *source_object,
+                GAsyncResult *res,
+                gpointer user_data)
+{
+  GVfsJobStartMountable *job = user_data;
+  GError *error;
+  GDrive *drive;
+
+  drive = G_DRIVE (source_object);
+
+  error = NULL;
+  if (g_drive_start_finish (drive, res, &error))
+    {
+      g_vfs_job_succeeded (G_VFS_JOB (job));
+    }
+  else
+    {
+      g_vfs_job_failed_from_error  (G_VFS_JOB (job), error);
+      g_error_free (error);
+    }
+}
+
+static gboolean
+try_start_mountable (GVfsBackend *backend,
+                     GVfsJobStartMountable *job,
+                     const char *filename,
+                     GMountSource *mount_source)
+{
+  ComputerFile *file;
+  GMountOperation *mount_op;
+
+  file = lookup (G_VFS_BACKEND_COMPUTER (backend),
+                 G_VFS_JOB (job), filename);
+
+  if (file == &root)
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_NOT_MOUNTABLE_FILE,
+                        _("Not a mountable file"));
+    }
+  else if (file != NULL)
+    {
+      if (file->drive != NULL)
+        {
+          mount_op = g_mount_source_get_operation (mount_source);
+          /* free mount_op when job is completed */
+          g_object_set_data_full (G_OBJECT (job), "gvfs-backend-computer-start-op", mount_op, g_object_unref);
+          g_drive_start (file->drive,
+                         0,
+                         mount_op,
+                         G_VFS_JOB (job)->cancellable,
+                         drive_start_cb,
+                         job);
+        }
+      else
+        {
+          g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                            G_IO_ERROR_NOT_SUPPORTED,
+                            _("Can't start file"));
+        }
+    }
+  else
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_NOT_SUPPORTED,
+                        _("Can't start file"));
+    }
+  return TRUE;
+}
+
+
+static void
+drive_stop_cb (GObject *source_object,
+               GAsyncResult *res,
+               gpointer user_data)
+{
+  GVfsJobStopMountable *job = user_data;
+  GError *error;
+  GDrive *drive;
+
+  drive = G_DRIVE (source_object);
+
+  error = NULL;
+  if (g_drive_stop_finish (drive, res, &error))
+    {
+      g_vfs_job_succeeded (G_VFS_JOB (job));
+    }
+  else
+    {
+      g_vfs_job_failed_from_error  (G_VFS_JOB (job), error);
+      g_error_free (error);
+    }
+}
+
+static gboolean
+try_stop_mountable (GVfsBackend *backend,
+                    GVfsJobStopMountable *job,
+                    const char *filename,
+                    GMountUnmountFlags flags,
+                    GMountSource *mount_source)
+{
+  ComputerFile *file;
+  GMountOperation *mount_op;
+
+  file = lookup (G_VFS_BACKEND_COMPUTER (backend),
+                 G_VFS_JOB (job), filename);
+
+  if (file == &root)
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_NOT_MOUNTABLE_FILE,
+                        _("Not a mountable file"));
+    }
+  else if (file != NULL)
+    {
+      if (file->drive != NULL)
+        {
+          mount_op = g_mount_source_get_operation (mount_source);
+          /* free mount_op when job is completed */
+          g_object_set_data_full (G_OBJECT (job), "gvfs-backend-computer-start-op", mount_op, g_object_unref);
+          g_drive_stop (file->drive,
+                        flags,
+                        mount_op,
+                        G_VFS_JOB (job)->cancellable,
+                        drive_stop_cb,
+                        job);
+        }
+      else
+        {
+          g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                            G_IO_ERROR_NOT_SUPPORTED,
+                            _("Can't stop file"));
+        }
+    }
+  else
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_NOT_SUPPORTED,
+                        _("Can't stop file"));
+    }
+  return TRUE;
+}
+
+static void
+drive_poll_for_media_cb (GObject *source_object,
+                         GAsyncResult *res,
+                         gpointer user_data)
+{
+  GVfsJobPollMountable *job = user_data;
+  GError *error;
+  GDrive *drive;
+
+  drive = G_DRIVE (source_object);
+
+  error = NULL;
+  if (g_drive_poll_for_media_finish (drive, res, &error))
+    {
+      g_vfs_job_succeeded (G_VFS_JOB (job));
+    }
+  else
+    {
+      g_vfs_job_failed_from_error  (G_VFS_JOB (job), error);
+      g_error_free (error);
+    }
+}
+
+static gboolean
+try_poll_mountable (GVfsBackend *backend,
+                    GVfsJobPollMountable *job,
+                    const char *filename)
+{
+  ComputerFile *file;
+
+  file = lookup (G_VFS_BACKEND_COMPUTER (backend),
+                 G_VFS_JOB (job), filename);
+
+  if (file == &root)
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_NOT_MOUNTABLE_FILE,
+                        _("Not a mountable file"));
+    }
+  else if (file != NULL)
+    {
+      if (file->drive != NULL)
+        {
+          g_drive_poll_for_media (file->drive,
+                                  G_VFS_JOB (job)->cancellable,
+                                  drive_poll_for_media_cb,
+                                  job);
+        }
+      else
+        {
+          g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                            G_IO_ERROR_NOT_SUPPORTED,
+                            _("Can't poll file"));
+        }
+    }
+  else
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_NOT_SUPPORTED,
+                        _("Can't poll file"));
+    }
+  return TRUE;
+}
+
 static void
 g_vfs_backend_computer_class_init (GVfsBackendComputerClass *klass)
 {
@@ -1122,4 +1414,7 @@ g_vfs_backend_computer_class_init (GVfsBackendComputerClass *klass)
   backend_class->try_mount_mountable = try_mount_mountable;
   backend_class->try_unmount_mountable = try_unmount_mountable;
   backend_class->try_eject_mountable = try_eject_mountable;
+  backend_class->try_start_mountable = try_start_mountable;
+  backend_class->try_stop_mountable = try_stop_mountable;
+  backend_class->try_poll_mountable = try_poll_mountable;
 }
