@@ -329,38 +329,70 @@ setup_async_wrapper (GMount *mount,
                      
 
 static void
-g_proxy_shadow_mount_eject (GMount              *mount,
-                            GMountUnmountFlags   flags,
-                            GCancellable        *cancellable,
-                            GAsyncReadyCallback  callback,
-                            gpointer             user_data)
+g_proxy_shadow_mount_eject_with_operation (GMount              *mount,
+                                           GMountUnmountFlags   flags,
+                                           GMountOperation     *mount_operation,
+                                           GCancellable        *cancellable,
+                                           GAsyncReadyCallback  callback,
+                                           gpointer             user_data)
 {
   GProxyShadowMount *proxy_shadow_mount = G_PROXY_SHADOW_MOUNT (mount);
   AsyncWrapperOp *data;
 
   data = setup_async_wrapper (mount, callback, user_data);
   G_LOCK (proxy_shadow_mount);
-  g_volume_eject (G_VOLUME (proxy_shadow_mount->volume),
-                  flags, cancellable,
-                  async_wrapper_callback, data);
+  g_volume_eject_with_operation (G_VOLUME (proxy_shadow_mount->volume),
+                                 flags, mount_operation, cancellable,
+                                 async_wrapper_callback, data);
   G_UNLOCK (proxy_shadow_mount);
 }
 
 static gboolean
-g_proxy_shadow_mount_eject_finish (GMount        *mount,
-                                   GAsyncResult  *result,
-                                   GError       **error)
+g_proxy_shadow_mount_eject_with_operation_finish (GMount        *mount,
+                                                  GAsyncResult  *result,
+                                                  GError       **error)
 {
   GProxyShadowMount *proxy_shadow_mount = G_PROXY_SHADOW_MOUNT (mount);
   gboolean res;
 
   G_LOCK (proxy_shadow_mount);
-  res = g_volume_eject_finish (G_VOLUME (proxy_shadow_mount->volume), result, error);
+  res = g_volume_eject_with_operation_finish (G_VOLUME (proxy_shadow_mount->volume), result, error);
   G_UNLOCK (proxy_shadow_mount);
 
   return res;
 }
 
+static void
+g_proxy_shadow_mount_unmount_with_operation (GMount              *mount,
+                                             GMountUnmountFlags   flags,
+                                             GMountOperation     *mount_operation,
+                                             GCancellable        *cancellable,
+                                             GAsyncReadyCallback  callback,
+                                             gpointer             user_data)
+{
+  GProxyShadowMount *proxy_shadow_mount = G_PROXY_SHADOW_MOUNT (mount);
+  AsyncWrapperOp *data;
+
+  data = setup_async_wrapper (mount, callback, user_data);
+  g_mount_unmount_with_operation (proxy_shadow_mount->real_mount,
+                                  flags,
+                                  mount_operation,
+                                  cancellable,
+                                  async_wrapper_callback, data);
+}
+
+static gboolean
+g_proxy_shadow_mount_unmount_with_operation_finish (GMount        *mount,
+                                                    GAsyncResult  *result,
+                                                    GError       **error)
+{
+  GProxyShadowMount *proxy_shadow_mount = G_PROXY_SHADOW_MOUNT (mount);
+
+  return g_mount_unmount_with_operation_finish (proxy_shadow_mount->real_mount,
+                                                result,
+                                                error);
+}
+
 static void
 g_proxy_shadow_mount_unmount (GMount              *mount,
                               GMountUnmountFlags   flags,
@@ -368,14 +400,7 @@ g_proxy_shadow_mount_unmount (GMount              *mount,
                               GAsyncReadyCallback  callback,
                               gpointer             user_data)
 {
-  GProxyShadowMount *proxy_shadow_mount = G_PROXY_SHADOW_MOUNT (mount);
-  AsyncWrapperOp *data;
-
-  data = setup_async_wrapper (mount, callback, user_data);
-  g_mount_unmount (proxy_shadow_mount->real_mount,
-                   flags,
-                   cancellable,
-                   async_wrapper_callback, data);
+  g_proxy_shadow_mount_unmount_with_operation (mount, flags, NULL, cancellable, callback, user_data);
 }
 
 static gboolean
@@ -383,11 +408,25 @@ g_proxy_shadow_mount_unmount_finish (GMount        *mount,
                                      GAsyncResult  *result,
                                      GError       **error)
 {
-  GProxyShadowMount *proxy_shadow_mount = G_PROXY_SHADOW_MOUNT (mount);
+  return g_proxy_shadow_mount_unmount_with_operation_finish (mount, result, error);
+}
 
-  return g_mount_unmount_finish (proxy_shadow_mount->real_mount,
-                                 result,
-                                 error);
+static void
+g_proxy_shadow_mount_eject (GMount              *mount,
+                            GMountUnmountFlags   flags,
+                            GCancellable        *cancellable,
+                            GAsyncReadyCallback  callback,
+                            gpointer             user_data)
+{
+  g_proxy_shadow_mount_eject_with_operation (mount, flags, NULL, cancellable, callback, user_data);
+}
+
+static gboolean
+g_proxy_shadow_mount_eject_finish (GMount        *mount,
+                                   GAsyncResult  *result,
+                                   GError       **error)
+{
+  return g_proxy_shadow_mount_eject_with_operation_finish (mount, result, error);
 }
 
 static void
@@ -457,8 +496,12 @@ g_proxy_shadow_mount_mount_iface_init (GMountIface *iface)
   iface->can_eject = g_proxy_shadow_mount_can_eject;
   iface->unmount = g_proxy_shadow_mount_unmount;
   iface->unmount_finish = g_proxy_shadow_mount_unmount_finish;
+  iface->unmount_with_operation = g_proxy_shadow_mount_unmount_with_operation;
+  iface->unmount_with_operation_finish = g_proxy_shadow_mount_unmount_with_operation_finish;
   iface->eject = g_proxy_shadow_mount_eject;
   iface->eject_finish = g_proxy_shadow_mount_eject_finish;
+  iface->eject_with_operation = g_proxy_shadow_mount_eject_with_operation;
+  iface->eject_with_operation_finish = g_proxy_shadow_mount_eject_with_operation_finish;
   iface->guess_content_type = g_proxy_shadow_mount_guess_content_type;
   iface->guess_content_type_finish = g_proxy_shadow_mount_guess_content_type_finish;
   iface->guess_content_type_sync = g_proxy_shadow_mount_guess_content_type_sync;
